#include "StdAfx.h"
#include "OpenArchive.h"
#include "UnAceV2J.h"
#include "Execute.h"
#include "ArchiveFileInfo.h"
#include "Tools.h"

static COpenArchive* pOATail;

COpenArchive::COpenArchive(void)
{
	m_pOANext = NULL;
	m_pOAPrev = pOATail;
	if (pOATail) {
		pOATail->m_pOANext = this;
	}
	pOATail = this;
}


COpenArchive::~COpenArchive(void)
{
	Close();
	if (m_pOAPrev) {
		m_pOAPrev->m_pOANext = m_pOANext;
	}
	if (m_pOANext) {
		m_pOANext->m_pOAPrev = m_pOAPrev;
	}
	if (pOATail == this) {
		pOATail = pOATail->m_pOAPrev;
	}
}

// ỸIuWFNg폜
void COpenArchive::RemoveAll()
{
	while (pOATail) {
		delete pOATail;
	}
}

// IuWFNg̃|C^
COpenArchive* COpenArchive::FindObject(HARC hArc)
{
	for (COpenArchive* pOA = pOATail; pOA; pOA = pOA->m_pOAPrev) {
		if (pOA == (COpenArchive*)hArc) {
			return pOA;
		}
	}
	return NULL;
}

BOOL COpenArchive::Open(const HWND _hwnd, LPCSTR lpFileName, DWORD dwMode)
{
	CExecute exec(_hwnd);
	m_bSearchMode = FALSE;
	m_dwMode = dwMode;
	CString strOption = "";
	m_aOSType = -1;
	// Ɏ̂̏擾B
	HANDLE hFind = ::FindFirstFile(lpFileName, &m_fdArchive);
	if (hFind == INVALID_HANDLE_VALUE) {
		return FALSE;
	}
	::FindClose(hFind);

	m_ArcFileName = lpFileName;
	m_WildName = "*";

	INT iResult = exec.Open(m_ArcFileName, m_WildName, this);
	if (iResult == 0) {
		if (m_ArcFileName.Right(4).CompareNoCase(".EXE") == 0) {
			m_IsSFX = TRUE;	
		} else {
			m_IsSFX = FALSE;
		}
		m_aOSType = exec.GetOSType();
		return TRUE;
	} else {
		return FALSE;
	}
}

int COpenArchive::FindFirst(LPCSTR lpszWildName, INDIVIDUALINFO *lpSubInfo)
{
	m_ui64ArcOriginalSize = 0;
	m_ui64ArcCompressedSize = 0;
	m_bSearchMode = TRUE;
	m_aItemPos = NULL;
	m_WildName = lpszWildName;
	m_aNumItems = 0;
	if (m_WildName.Compare("*.*") == 0) {
		m_WildName = "*";
	}

	if (m_list.GetCount() > 0) {
		m_aItemPos = m_list.GetHeadPosition();
		return FindNext(lpSubInfo);
	} else {
		m_IsSFX = FALSE;
		return -1;	// I
	}
}

int COpenArchive::FindNext(INDIVIDUALINFO *lpSubInfo)
{
	CArchiveFileInfo* afi;
	FILETIME ftLocalTime;

	if (!m_bSearchMode) {
		return ERROR_NOT_SEARCH_MODE;
	}

	if (m_aItemPos == NULL) {
		return -1;	// I
	}
	while (TRUE) {
		afi = (CArchiveFileInfo *)m_list.GetAt(m_aItemPos);
		CString strFilename = afi->GetFilename();
		m_list.GetNext(m_aItemPos);
		if (CTools::IsMatch(strFilename, m_WildName)) {
			break;
		} else {
			if (m_aItemPos == NULL) {
				return -1;	// I
			}
		}
	}

	m_aFileName = afi->GetFilename(); 
	if (afi->GetAttr() & FILE_ATTRIBUTE_DIRECTORY) {
		m_aFileName.TrimRight('\\');
		m_aFileName += "\\";
	}

	m_ui64OriginalSize = afi->GetLength();

	m_ui64CompressedSize = afi->GetCompressedLength();

	m_dwCRC = afi->GetCRC32();

	m_ftLastWriteTime = afi->GetModificationDateTime();

	m_dwAttribute = afi->GetAttr(); 

	m_aMode = afi->GetMethod();

	if (lpSubInfo) {
		lpSubInfo->uFlag = 0;
		lpSubInfo->uOSType = 0;
		lpSubInfo->dwCRC = m_dwCRC;
		lpSubInfo->dwOriginalSize = (DWORD)m_ui64OriginalSize;
		lpSubInfo->dwCompressedSize = (DWORD)m_ui64CompressedSize;
		lpSubInfo->wRatio = m_ui64OriginalSize ? (WORD)(m_ui64CompressedSize * 1000 / m_ui64OriginalSize) : 0;
		::FileTimeToLocalFileTime(&m_ftLastWriteTime, &ftLocalTime); 
		::FileTimeToDosDateTime(&ftLocalTime, &lpSubInfo->wDate, &lpSubInfo->wTime);
		::lstrcpynA(lpSubInfo->szFileName, m_aFileName, FNAME_MAX32 + 1);
		::lstrcpynA(lpSubInfo->szMode, m_aMode, 8);
		lpSubInfo->szAttribute[0] = (m_dwAttribute & FILE_ATTRIBUTE_ARCHIVE)  ? 'A' : '-';
		lpSubInfo->szAttribute[1] = (m_dwAttribute & FILE_ATTRIBUTE_SYSTEM)   ? 'S' : '-';
		lpSubInfo->szAttribute[2] = (m_dwAttribute & FILE_ATTRIBUTE_HIDDEN)   ? 'H' : '-';
		lpSubInfo->szAttribute[3] = (m_dwAttribute & FILE_ATTRIBUTE_READONLY) ? 'R' : 'W';
		lpSubInfo->szAttribute[4] = '\0';
	}

	m_ui64ArcOriginalSize += m_ui64OriginalSize;
	m_ui64ArcCompressedSize += m_ui64CompressedSize;
	m_aNumItems ++;
	return 0;
}

int COpenArchive::GetArcFileName(LPSTR lpBuffer, const int nSize)
{
	::lstrcpynA(lpBuffer, m_fdArchive.cFileName, nSize);
	return 0;
}

DWORD COpenArchive::GetArcFileSize(void)
{
	return m_fdArchive.nFileSizeLow;
}

DWORD COpenArchive::GetArcOriginalSize(void)
{
	return (DWORD)m_ui64ArcOriginalSize;
}

DWORD COpenArchive::GetArcCompressedSize(void)
{
	return (DWORD)m_ui64ArcCompressedSize;
}

WORD COpenArchive::GetArcRatio(void)
{
	if (m_ui64ArcOriginalSize == 0) {
		return 0;
	}
	return (WORD)(m_ui64ArcCompressedSize * 1000 / m_ui64ArcOriginalSize);
}
WORD COpenArchive::GetArcDate(void)
{
	WORD wFatDate;
	WORD wFatTime;
	FILETIME ftLocalFileTime;
	::FileTimeToLocalFileTime(&m_fdArchive.ftLastWriteTime, &ftLocalFileTime);
	::FileTimeToDosDateTime(&ftLocalFileTime, &wFatDate, &wFatTime);
	return wFatDate;
}

WORD COpenArchive::GetArcTime(void)
{
	WORD wFatDate;
	WORD wFatTime;
	FILETIME ftLocalFileTime;
	::FileTimeToLocalFileTime(&m_fdArchive.ftLastWriteTime, &ftLocalFileTime);
	::FileTimeToDosDateTime(&ftLocalFileTime, &wFatDate, &wFatTime);
	return wFatTime;
}

UINT COpenArchive::GetArcOSType(void)
{
	return m_aOSType;
}

int COpenArchive::IsSFXFile(void)
{
	return m_IsSFX;
}

int COpenArchive::GetFileName(LPSTR lpBuffer, const int nSize)
{
	if (!m_bSearchMode) {
		return ERROR_NOT_SEARCH_MODE;
	}
	::lstrcpynA(lpBuffer, m_aFileName, nSize);
	return 0;
}

DWORD COpenArchive::GetOriginalSize(void)
{
	if (!m_bSearchMode) {
		return -1;
	}
	return (DWORD)m_ui64OriginalSize;
}

DWORD COpenArchive::GetCompressedSize(void)
{
	if (!m_bSearchMode) {
		return -1;
	}
	return (DWORD)m_ui64CompressedSize;
}

WORD COpenArchive::GetRatio(void)
{
	if (!m_bSearchMode) {
		return -1;
	}
	if (m_ui64OriginalSize == 0) {
		return 0;
	}
	return (WORD)(m_ui64CompressedSize * 1000 / m_ui64OriginalSize);
}

WORD COpenArchive::GetDate(void)
{
	if (!m_bSearchMode) {
		return -1;
	}
	WORD wFatDate;
	WORD wFatTime;
	FILETIME ftLocalFileTime;
	::FileTimeToLocalFileTime(&m_ftLastWriteTime, &ftLocalFileTime);
	::FileTimeToDosDateTime(&ftLocalFileTime, &wFatDate, &wFatTime);
	return wFatDate;
}

WORD COpenArchive::GetTime(void)
{
	if (!m_bSearchMode) {
		return -1;
	}
	WORD wFatDate;
	WORD wFatTime;
	FILETIME ftLocalFileTime;
	::FileTimeToLocalFileTime(&m_ftLastWriteTime, &ftLocalFileTime);
	::FileTimeToDosDateTime(&ftLocalFileTime, &wFatDate, &wFatTime);
	return wFatTime;
}

DWORD COpenArchive::GetCRC(void)
{
	if (!m_bSearchMode) {
		return -1;
	}
	return m_dwCRC;
}

int COpenArchive::GetAttribute(void)
{
	if (!m_bSearchMode) {
		return -1;
	}
	return m_dwAttribute & 0xff;
}

UINT COpenArchive::GetOSType(void)
{
	if (!m_bSearchMode) {
		return -1;
	}
	return m_aOSType;
}

int COpenArchive::GetMethod(LPSTR lpBuffer, const int nSize)
{
	if (!m_bSearchMode) {
		return ERROR_NOT_SEARCH_MODE;
	}
	::lstrcpynA(lpBuffer, m_aMode, nSize);
	return 0;
}

DWORD COpenArchive::GetWriteTime(void)
{
	if (!m_bSearchMode) {
		return -1;
	}
	struct tm atm;
	FILETIME ftLocalFileTime;
	SYSTEMTIME stLastWriteTime;
	::FileTimeToLocalFileTime(&m_ftLastWriteTime, &ftLocalFileTime);
	::FileTimeToSystemTime(&ftLocalFileTime, &stLastWriteTime);
	atm.tm_year = stLastWriteTime.wYear - 1900;
	atm.tm_mon = stLastWriteTime.wMonth - 1;
	atm.tm_mday = stLastWriteTime.wDay;
	atm.tm_hour = stLastWriteTime.wHour;
	atm.tm_min = stLastWriteTime.wMinute;
	atm.tm_sec = stLastWriteTime.wSecond;
	atm.tm_isdst = -1;
	return (DWORD)mktime(&atm);
}

DWORD COpenArchive::GetCreateTime(void)
{
	if (!m_bSearchMode) {
		return -1;
	}
	struct tm atm;
	FILETIME ftLocalFileTime;
	SYSTEMTIME stLastCreateTime;
	::FileTimeToLocalFileTime(&m_ftLastCreateTime, &ftLocalFileTime);
	::FileTimeToSystemTime(&ftLocalFileTime, &stLastCreateTime);
	atm.tm_year = stLastCreateTime.wYear - 1900;
	atm.tm_mon = stLastCreateTime.wMonth - 1;
	atm.tm_mday = stLastCreateTime.wDay;
	atm.tm_hour = stLastCreateTime.wHour;
	atm.tm_min = stLastCreateTime.wMinute;
	atm.tm_sec = stLastCreateTime.wSecond;
	atm.tm_isdst = -1;
	return (DWORD)mktime(&atm);
}

DWORD COpenArchive::GetAccessTime(void)
{
	if (!m_bSearchMode) {
		return -1;
	}
	struct tm atm;
	FILETIME ftLocalFileTime;
	SYSTEMTIME stLastAccessTime;
	::FileTimeToLocalFileTime(&m_ftLastAccessTime, &ftLocalFileTime);
	::FileTimeToSystemTime(&ftLocalFileTime, &stLastAccessTime);
	atm.tm_year = stLastAccessTime.wYear - 1900;
	atm.tm_mon = stLastAccessTime.wMonth - 1;
	atm.tm_mday = stLastAccessTime.wDay;
	atm.tm_hour = stLastAccessTime.wHour;
	atm.tm_min = stLastAccessTime.wMinute;
	atm.tm_sec = stLastAccessTime.wSecond;
	atm.tm_isdst = -1;
	return (DWORD)mktime(&atm);
}

BOOL COpenArchive::GetWriteTimeEx(FILETIME *lpftLastWriteTime)
{
	if (!m_bSearchMode) {
		return FALSE;
	}
	*lpftLastWriteTime = m_ftLastWriteTime;
	return TRUE;
}

BOOL COpenArchive::GetCreateTimeEx(FILETIME *lpftLastCreateTime)
{
	if (!m_bSearchMode) {
		return FALSE;
	}
	*lpftLastCreateTime = m_ftLastCreateTime;
	return TRUE;
}

BOOL COpenArchive::GetAccessTimeEx(FILETIME *lpftLastAccessTime)
{
	if (!m_bSearchMode) {
		return FALSE;
	}
	*lpftLastAccessTime = m_ftLastAccessTime;
	return TRUE;
}

BOOL COpenArchive::GetArcCreateTimeEx(FILETIME *lpftCreationTime)
{
	*lpftCreationTime = m_fdArchive.ftCreationTime;
	return TRUE;
}

BOOL COpenArchive::GetArcAccessTimeEx(FILETIME *lpftLastAccessTime)
{
	*lpftLastAccessTime = m_fdArchive.ftLastAccessTime;
	return TRUE;
}

BOOL COpenArchive::GetArcWriteTimeEx(FILETIME *lpftLastWriteTime)
{
	*lpftLastWriteTime = m_fdArchive.ftLastWriteTime;
	return TRUE;
}

BOOL COpenArchive::GetArcFileSizeEx(ULHA_INT64* lpllSize)
{
	*lpllSize = ((ULHA_INT64)m_fdArchive.nFileSizeHigh << 32) + m_fdArchive.nFileSizeLow;
	return TRUE;
}

BOOL COpenArchive::GetArcOriginalSizeEx(ULHA_INT64* lpllSize)
{
	*lpllSize = (ULHA_INT64)m_ui64ArcOriginalSize;
	return TRUE;
}

BOOL COpenArchive::GetArcCompressedSizeEx(ULHA_INT64* lpllSize)
{
	*lpllSize = (ULHA_INT64)m_ui64ArcCompressedSize;
	return TRUE;
}

BOOL COpenArchive::GetOriginalSizeEx(ULHA_INT64* lpllSize)
{
	if (!m_bSearchMode) {
		return FALSE;
	}
	*lpllSize = (ULHA_INT64)m_ui64OriginalSize;
	return TRUE;
}

BOOL COpenArchive::GetCompressedSizeEx(ULHA_INT64* lpllSize)
{
	if (!m_bSearchMode) {
		return FALSE;
	}
	*lpllSize = (ULHA_INT64)m_ui64CompressedSize;
	return TRUE;
}

int COpenArchive::GetFileCount()
{
	return m_aNumItems;
}

void COpenArchive::Close(void)
{
	CArchiveFileInfo* afi;

	while (!m_list.IsEmpty()) {
		afi = (CArchiveFileInfo *)m_list.GetTail();
		delete afi;
		m_list.RemoveTail();
	}
}
